unit MyRVStyle;

interface

{$I RV_Defs.inc}

uses
     {$IFDEF RICHVIEWDEF2009}AnsiStrings,{$ENDIF}
     SysUtils, Classes, Windows, Graphics, Controls, RVStyle, RVTypes, RVScroll,
     RVGrIn, RVFuncs, RVClasses, RVFMisc, RVRTF, RVRTFProps;

type

  { TMyFontInfo - class of TMyRVStyle.TextStyles[]
    It has one new property - Shadow.
    In TRVFontInfoProperties, it corresponds to rvfiCustom (used in IsEqual())

    This method has overriden methods for drawing, saving to HTML CSS,
    saving and loading from INI-file, saving to RTF.

    Required version of TRichView 14+

    Not supported: style templates
  }

  TMyFontInfo = class (TFontInfo)
  private
    FShadow: Boolean;
  protected
    function IsSimpleEqual(Value: TCustomRVInfo; IgnoreReferences: Boolean;
      IgnoreID: Boolean=True): Boolean; override;
    function SimilarityValue(Value: TCustomRVInfo): Integer; override;
  public
    procedure Assign(Source: TPersistent); override;
    function IsEqual(Value: TCustomRVFontInfo; IgnoreList: TRVFontInfoProperties): Boolean; override;
    procedure SaveToINI(ini: TRVIniFile; const Section, fs: String;
      Properties: TRVFontInfoProperties); override;
    procedure LoadFromINI(ini: TRVIniFile; const Section, fs: String;
      JumpByDefault: Boolean; DefJumpCursor: TCursor); override;
    procedure Draw(const s: TRVRawByteString; Canvas: TCanvas; ThisStyleNo: Integer;
      SpaceAtLeft, Left, Top, Width, Height, BaseLine: Integer;
      RVStyle: TRVStyle;
      DrawState: TRVTextDrawStates; Printing, PreviewCorrection: Boolean;
      ColorMode: TRVColorMode; DefBiDiMode: TRVBiDiMode;
      RefCanvas: TCanvas; GraphicInterface: TRVGraphicInterface); override;
    procedure SaveCSSToStream(Stream: TStream; BaseStyle: TCustomRVFontInfo;
      Multiline, UTF8, Jump: Boolean); override;
    procedure SaveRTFToStream(Stream: TStream; StyleToFont: TRVIntegerList;
      RTFTables: TObject; RVStyle: TRVStyle; InheritedStyle: Boolean); override;
  published
    property Shadow: Boolean read FShadow write FShadow default False;
  end;
  {
   TMyRVStyle - TRVStyle having a changed type of items in TextStyles
  }
  TMyRVStyle = class (TRVStyle)
  public
    function GetTextStyleClass: TRVFontInfoClass; override;
  end;
  {
  TMyRTFCharProperties - class containing character properties read from RTF.
  This class is assigned to RTFCharPropertiesClass global variable
  (in the initialization section of this unit)
  }
  TMyRTFCharProperties = class (TRVRTFCharProperties)
  private
    FShadow: Boolean;
  public
    procedure Reset (DefLanguage: Cardinal; DefFontIndex: Integer); override;
    procedure Assign(Source: TRVRTFCharProperties); override;
    function CanParseKeyword(const Keyword: TRVAnsiString): Boolean; override;
    procedure ParseKeyword(const Keyword: TRVAnsiString; param: Integer;
         fParam:Boolean); override;
    property Shadow: Boolean read FShadow;
  end;
  {
  TMyRTFStyleComparer - a link between TMyFontInfo and TMyRTFCharProperties.
  An object of this class must be assigned to RichView.RTFReadProperties.StyleComparer
  }
  TMyRTFStyleComparer = class (TRVRTFStyleComparer)
    public
      function AreMatched(TextStyle: TFontInfo; RTFCharProps: TRVRTFCharProperties): Boolean; override;
      function SimilarityValue(TextStyle: TCustomRVFontInfo; RTFCharProps: TRVRTFCharProperties): Integer; override;
      procedure AssignFromRTF(TextStyle: TCustomRVFontInfo; RTFCharProps: TRVRTFCharProperties); override;
  end;

const SHADOW_COLOR = clGray;
      SHADOW_OFFS = 2;

implementation
uses RVMapWht;
{============================== TMyFontInfo ===================================}
procedure TMyFontInfo.Assign(Source: TPersistent);
begin
  if Source is TMyFontInfo then begin
    Shadow := TMyFontInfo(Source).Shadow;
  end;
  inherited Assign(Source)
end;
{------------------------------------------------------------------------------}
procedure TMyFontInfo.Draw(const s: TRVRawByteString; Canvas: TCanvas;
  ThisStyleNo, SpaceAtLeft, Left, Top, Width, Height, BaseLine: Integer;
  RVStyle: TRVStyle; DrawState: TRVTextDrawStates; Printing,
  PreviewCorrection: Boolean; ColorMode: TRVColorMode;
  DefBiDiMode: TRVBiDiMode; RefCanvas: TCanvas;
  GraphicInterface: TRVGraphicInterface);
var LColor: TColor;
begin
  if Shadow and not (rvtsSelected in DrawState)
    and (ColorMode in [rvcmColor, rvcmPrinterColor, rvcmGrayScale]) then begin
    LColor := Canvas.Font.Color;
    case ColorMode of
      rvcmColor:
        Canvas.Font.Color := SHADOW_COLOR;
      rvcmPrinterColor:
        Color := RV_GetPrnColor(SHADOW_COLOR);
      else //rvcmGrayScale:
        Color := RV_GetGray(RV_GetPrnColor(SHADOW_COLOR));
    end;
    Shadow := False;
    try
      Draw(s, Canvas, ThisStyleNo, SpaceAtLeft, Left+SHADOW_OFFS, Top+SHADOW_OFFS,
        Width, Height, BaseLine,
        RVStyle, DrawState, Printing, PreviewCorrection, ColorMode, DefBiDiMode,
        RefCanvas, GraphicInterface);
    finally
      Shadow := True;
      Canvas.Font.Color := LColor;
    end;
  end;
  inherited;
end;
{------------------------------------------------------------------------------}
function TMyFontInfo.IsEqual(Value: TCustomRVFontInfo;
  IgnoreList: TRVFontInfoProperties): Boolean;
begin
  Result := inherited IsEqual(Value, IgnoreList);
  if Result and (Value is TMyFontInfo) then
    Result := ((rvfiCustom in IgnoreList) or (Shadow = TMyFontInfo(Value).Shadow));
end;
{------------------------------------------------------------------------------}
function TMyFontInfo.IsSimpleEqual(Value: TCustomRVInfo; IgnoreReferences,
  IgnoreID: Boolean): Boolean;
begin
  Result := inherited IsSimpleEqual(Value, IgnoreReferences, IgnoreID);
  if Result and (Value is TMyFontInfo) then
    Result := (Shadow = TMyFontInfo(Value).Shadow);
end;
{------------------------------------------------------------------------------}
procedure TMyFontInfo.LoadFromINI(ini: TRVIniFile; const Section,
  fs: String; JumpByDefault: Boolean; DefJumpCursor: TCursor);
begin
  inherited;
  Shadow := IniReadBool(ini, Section, Format(fs,['Shadow']), False);
end;
{------------------------------------------------------------------------------}
procedure TMyFontInfo.SaveToINI(ini: TRVIniFile; const Section, fs: String;
  Properties: TRVFontInfoProperties);
begin
  inherited;
  if rvfiCustom in Properties then
    WriteBoolToIniIfNE(ini, Section, Format(fs,['Shadow']), Shadow, False);
end;
{------------------------------------------------------------------------------}
procedure TMyFontInfo.SaveCSSToStream(Stream: TStream;
  BaseStyle: TCustomRVFontInfo; Multiline, UTF8, Jump: Boolean);
begin
  inherited;
  // note: "text-shadow" is not supported by IE
  if (BaseStyle=nil) or (TMyFontInfo(BaseStyle).Shadow<>Shadow) then
    if Shadow then
      RVWriteX(Stream, {$IFDEF RVUNICODESTR}AnsiStrings.{$ENDIF}
        Format(' text-shadow: %s %dpx %dpx 0px;',[RV_GetCSSBkColor(SHADOW_COLOR),
          SHADOW_OFFS, SHADOW_OFFS]),
        Multiline)
    else if (BaseStyle<>nil) then
      RVWriteX(Stream, ' text-shadow: none;', Multiline);
end;
{------------------------------------------------------------------------------}
function TMyFontInfo.SimilarityValue(Value: TCustomRVInfo): Integer;
begin
  Result := inherited SimilarityValue(Value);
  if (Value is TMyFontInfo) then begin
    if TMyFontInfo(Value).Shadow=Shadow then
      inc(Result, RVSMW_FONTEACHSTYLE div 2);
  end;
end;
{------------------------------------------------------------------------------}
procedure TMyFontInfo.SaveRTFToStream(Stream: TStream; StyleToFont: TRVIntegerList;
  RTFTables: TObject; RVStyle: TRVStyle; InheritedStyle: Boolean);
begin
  inherited;
  if Shadow then
    RVFWrite(Stream, '\shad');
end;
{================================ TMyRVStyle ==================================}
function TMyRVStyle.GetTextStyleClass: TRVFontInfoClass;
begin
  Result := TMyFontInfo;
end;
{========================= TMyRTFCharProperties ===============================}
procedure TMyRTFCharProperties.Assign(Source: TRVRTFCharProperties);
begin
  inherited Assign(Source);
  if Source is TMyRTFCharProperties then begin
    FShadow := TMyRTFCharProperties(Source).Shadow;
  end;
end;
{------------------------------------------------------------------------------}
function TMyRTFCharProperties.CanParseKeyword(const Keyword: TRVAnsiString): Boolean;
begin
  Result := (Keyword='shad');
end;
{------------------------------------------------------------------------------}
procedure TMyRTFCharProperties.ParseKeyword(const Keyword: TRVAnsiString;
  param: Integer; fParam: Boolean);
begin
  if (Keyword='shad') then
    FShadow := (not fParam) or (param<>0);
end;
{------------------------------------------------------------------------------}
procedure TMyRTFCharProperties.Reset(DefLanguage: Cardinal;
  DefFontIndex: Integer);
begin
  inherited Reset(DefLanguage, DefFontIndex);
  FShadow := False;
end;
{=========================== TMyRTFStyleComparer ==============================}
function TMyRTFStyleComparer.AreMatched(TextStyle: TFontInfo;
  RTFCharProps: TRVRTFCharProperties): Boolean;
begin
  if (TextStyle is TMyFontInfo) and (RTFCharProps is TMyRTFCharProperties) then
    Result := TMyFontInfo(TextStyle).Shadow=TMyRTFCharProperties(RTFCharProps).Shadow
  else
    Result := True;
end;
{------------------------------------------------------------------------------}
procedure TMyRTFStyleComparer.AssignFromRTF(TextStyle: TCustomRVFontInfo;
  RTFCharProps: TRVRTFCharProperties);
begin
if (TextStyle is TMyFontInfo) and (RTFCharProps is TMyRTFCharProperties) then
    TMyFontInfo(TextStyle).Shadow := TMyRTFCharProperties(RTFCharProps).Shadow;
end;
{------------------------------------------------------------------------------}
function TMyRTFStyleComparer.SimilarityValue(TextStyle: TCustomRVFontInfo;
  RTFCharProps: TRVRTFCharProperties): Integer;
begin
  Result := 0;
  if (TextStyle is TMyFontInfo) and (RTFCharProps is TMyRTFCharProperties) then
    if TMyFontInfo(TextStyle).Shadow=TMyRTFCharProperties(RTFCharProps).Shadow then
      inc(Result, RVSMW_FONTEACHSTYLE div 2);
end;

initialization
  RTFCharPropertiesClass := TMyRTFCharProperties;

end.
